@@ -20,7 +20,7 @@ except ImportError:
     print('"click" not found, commands unavailable', file=sys.stderr)
     sys.exit(1)
 
-from .exceptions import FrameCRCMismatch
+from .exceptions import FrameCRCMismatch, FrameLengthExceeded, InvalidCommand
 from .frame import ReceiveFrame, make_frame
 from .registry import REGISTRY as R
 from .simulator import run_simulator
@@ -34,7 +34,8 @@ log = logging.getLogger('rctclient.cli')
 @click.pass_context
 @click.version_option()
 @click.option('-d', '--debug', is_flag=True, default=False, help='Enable debug output')
-def cli(ctx, debug: bool) -> None:
+@click.option('--frame-debug', is_flag=True, default=False, help='Enables frame debugging (requires --debug)')
+def cli(ctx, debug: bool, frame_debug: bool) -> None:
     '''
     rctclient toolbox. Please help yourself with the subcommands.
     '''
@@ -46,6 +47,8 @@ def cli(ctx, debug: bool) -> None:
             level=logging.DEBUG,
             format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
         )
+        if not frame_debug:
+            logging.getLogger('rctclient.frame.ReceiveFrame').setLevel(logging.INFO)
     log.info('rctclient CLI starting')
 
 
@@ -75,19 +78,19 @@ def receive_frame(sock: socket.socket, timeout: int = 2) -> ReceiveFrame:
         try:
             ready_read, _, _ = select.select([sock], [], [], timeout)
         except select.error as exc:
-            log.error(f'Error during receive: select returned an error: {str(exc)}')
+            log.error('Error during receive: select returned an error: %s', str(exc))
             raise
 
         if ready_read:
             buf = sock.recv(1024)
             if len(buf) > 0:
-                log.debug(f'Received {len(buf)} bytes: {buf.hex()}')
+                log.debug('Received %d bytes: %s', len(buf), buf.hex())
                 i = frame.consume(buf)
-                log.debug(f'Frame consumed {i} bytes')
+                log.debug('Frame consumed %d bytes', i)
                 if frame.complete():
                     if len(buf) > i:
-                        log.warning(f'Frame complete, but buffer still contains {len(buf) - i} bytes')
-                        log.debug(f'Leftover bytes: {buf[i:].hex()}')
+                        log.warning('Frame complete, but buffer still contains %d bytes', len(buf) - i)
+                        log.debug('Leftover bytes: %s', buf[i:].hex())
                     return frame
     raise TimeoutError
 
@@ -108,12 +111,12 @@ def read_value(ctx, port: int, host: str, id: Optional[str], name: Optional[str]
     response is returned on stdout. Without "verbose" set, the value is returned on standard out, otherwise more
     information about the object is printed with the value.
 
-    Specify either "<id>" or "<name>". The ID must be in the form '0xABCD', the name must exactly match the name of a
-    known object ID including the group prefix.
+    Specify either "--id <id>" or "--name <name>". The ID must be in th decimal notation, such as "0x959930BF", the
+    name must exactly match the name of a known object ID such as "battery.soc".
 
     The "<name>" option supports shell autocompletion (if installed).
 
-    If "debug" is set, log output is sent to stderr, so the value can be read from stdout while still catching
+    If "--debug" is set, log output is sent to stderr, so the value can be read from stdout while still catching
     everything else on stderr.
 
     Timeseries data and the event table will be queried using the current time. Note that the device may send an
@@ -124,8 +127,8 @@ def read_value(ctx, port: int, host: str, id: Optional[str], name: Optional[str]
     Examples:
 
     \b
-    rctclient read-value --name temperature.sink_temp_power_reduction
-    rctclient read-value --id 0x90B53336
+    rctclient read-value --host 192.168.0.1 --name temperature.sink_temp_power_reduction
+    rctclient read-value --host 192.168.0.1 --id 0x90B53336
     \f
     :param ctx: Click context
     :param port: The port number.
@@ -141,17 +144,17 @@ def read_value(ctx, port: int, host: str, id: Optional[str], name: Optional[str]
     try:
         if id:
             real_id = int(id[2:], 16)
-            log.debug(f'Parsed ID: 0x{real_id:X}')
+            log.debug('Parsed ID: 0x%X', real_id)
             oinfo = R.get_by_id(real_id)
-            log.debug(f'Object info by ID: {oinfo}')
+            log.debug('Object info by ID: %s', oinfo)
         elif name:
             oinfo = R.get_by_name(name)
-            log.debug(f'Object info by name: {oinfo}')
+            log.debug('Object info by name: %s', oinfo)
     except KeyError:
         log.error('Could not find requested id or name')
         sys.exit(1)
     except ValueError as exc:
-        log.debug(f'Invalid --id parameter: {str(exc)}')
+        log.debug('Invalid --id parameter: %s', str(exc))
         log.error('Invalid --id parameter, can\'t parse', err=True)
         sys.exit(1)
 
@@ -159,9 +162,9 @@ def read_value(ctx, port: int, host: str, id: Optional[str], name: Optional[str]
     try:
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock.connect((host, port))
-        log.debug(f'Connected to {host}:{port}')
+        log.debug('Connected to %s:%d', host, port)
     except socket.error as exc:
-        log.error(f'Could not connect to host: {str(exc)}')
+        log.error('Could not connect to host: %s', str(exc))
         sys.exit(1)
 
     is_ts = oinfo.response_data_type == DataType.TIMESERIES
@@ -174,12 +177,19 @@ def read_value(ctx, port: int, host: str, id: Optional[str], name: Optional[str]
     try:
         rframe = receive_frame(sock)
     except FrameCRCMismatch as exc:
-        log.error(f'Received frame CRC mismatch: received 0x{exc.received_crc:X} but calculated '
-                  f'0x{exc.calculated_crc:X}')
+        log.error('Received frame CRC mismatch: received 0x%X but calculated 0x%X',
+                  exc.received_crc, exc.calculated_crc)
         sys.exit(1)
-    log.debug(f'Got frame: {rframe}')
+    except InvalidCommand:
+        log.error('Received an unexpected/invalid command in response')
+        sys.exit(1)
+    except FrameLengthExceeded:
+        log.error('Parser overshot, cannot recover frame')
+        sys.exit(1)
+
+    log.debug('Got frame: %s', rframe)
     if rframe.id != oinfo.object_id:
-        log.error(f'Received unexpected frame, ID is 0x{rframe.id:X}, expected 0x{oinfo.object_id:X}')
+        log.error('Received unexpected frame, ID is 0x%X, expected 0x%X', rframe.id, oinfo.object_id)
         sys.exit(1)
 
     if is_ts or is_ev:
@@ -194,7 +204,11 @@ def read_value(ctx, port: int, host: str, id: Optional[str], name: Optional[str]
                 e4 = f'0x{entry.element4:x}' if entry.element4 is not None else ''
                 value += f'0x{entry.entry_type:x},{entry.timestamp:%Y-%m-%dT%H:%M:%S},{e2},{e3},{e4}\n'
     else:
-        value = decode_value(oinfo.response_data_type, rframe.data)
+        # hexdump if the data type is now known
+        if oinfo.response_data_type == DataType.UNKNOWN:
+            value = '0x' + rframe.data.hex()
+        else:
+            value = decode_value(oinfo.response_data_type, rframe.data)
 
     if verbose:
         description = oinfo.description if oinfo.description is not None else ''
@@ -206,8 +220,8 @@ def read_value(ctx, port: int, host: str, id: Optional[str], name: Optional[str]
 
     try:
         sock.close()
-    except Exception as exc:
-        log.error(f'Exception when disconnecting from the host: {str(exc)}')
+    except Exception as exc:  # pylint: disable=broad-except
+        log.error('Exception when disconnecting from the host: %s', str(exc))
     sys.exit(0)
 
 
