@@ -1,8 +1,12 @@
 
 # Copyright 2020, Peter Oberhofer (pob90)
-# Copyright 2020, Stefan Valouch (svalouch)
+# Copyright 2020-2021, Stefan Valouch (svalouch)
 # SPDX-License-Identifier: GPL-3.0-only
 
+'''
+Type declarations.
+'''
+
 # pylint: disable=too-many-arguments,too-few-public-methods
 
 from datetime import datetime
@@ -19,18 +23,42 @@ class Command(IntEnum):
     READ = 0x01
     #: Write command
     WRITE = 0x02
-    #: Long write command
+    #: Long write command (use for variables > 251 bytes)
     LONG_WRITE = 0x03
     #: Response to a read or write command
     RESPONSE = 0x05
-    #: Long response
+    #: Long response (for variables > 251 bytes)
     LONG_RESPONSE = 0x06
+    # Periodic reading
+    # READ_PERIODICALLY = 0x08
+
+    #: Plant: Read command
+    PLANT_READ = READ | 0x40
+    #: Plant: Write command
+    PLANT_WRITE = WRITE | 0x40
+    #: Plant: Long write
+    PLANT_LONG_WRITE = LONG_WRITE | 0x40
+
     #: Extension
     EXTENSION = 0x3c
 
     #: Sentinel, do not use
     _NONE = 0xff
 
+    @staticmethod
+    def is_plant(command: 'Command') -> bool:
+        '''
+        Returns whether a command is for plant communication by checking if bit 6 is set.
+        '''
+        return bool(command & 0x40)
+
+    @staticmethod
+    def is_long(command: 'Command') -> bool:
+        '''
+        Returns whether a command is a long command.
+        '''
+        return command in (Command.LONG_WRITE, Command.LONG_RESPONSE, Command.PLANT_LONG_WRITE)
+
 
 class ObjectGroup(IntEnum):
     '''
@@ -57,7 +85,7 @@ class ObjectGroup(IntEnum):
     POWER_MNG = 17
     BUF_V_CONTROL = 18
     DB = 19
-    SWITCH_ON_BOARD = 20
+    SWITCH_ON_COND = 20
     P_REC = 21
     MODBUS = 22
     BAT_MNG_STRUCT = 23
@@ -72,6 +100,8 @@ class ObjectGroup(IntEnum):
     LINE_MON = 32
     OTHERS = 33
     BATTERY_PLACEHOLDER = 34
+    FRT = 35
+    PARTITION = 36
 
 
 class FrameType(IntEnum):
@@ -83,6 +113,9 @@ class FrameType(IntEnum):
     #: Plant frame with ID and address
     PLANT = 8
 
+    #: Sentinel value, denotes unknown type.
+    _NONE = 0
+
 
 class DataType(IntEnum):
     '''
